/* 	DNR.c - DNR library for MPW

	(c) Copyright 1988 by Apple Computer.  All rights reserved
	Modified for ThinkC 4 by Mark C Smith University of Michigan ITD RS UP
	
*/

#include <OSUtils.h>
#include <Memory.h>
#include <Files.h>
#include <Resources.h>
#include <Errors.h>
#include "tcp.h"

#define OPENRESOLVER	1L
#define CLOSERESOLVER	2L
#define STRTOADDR		3L
#define	ADDRTOSTR		4L
#define	ENUMCACHE		5L
#define ADDRTONAME		6L

Handle codeHndl = nil;

typedef OSErr (*DNRProcPtr)( long, ... );
DNRProcPtr dnr = nil;

/*
 * prototypes to satisfy the compiler
 */
short OpenOurRF( void );
OSErr CloseResolver( void );

/*
 * prototypes
 */
short OpenOurRF( void );
OSErr CloseResolver( void );
OSErr StrToAddr( char *hostName, struct hostInfo *rtnStruct, ResultProcPtr resultproc,
					char *userDataPtr );
OSErr AddrToStr( unsigned long addr, char *addrStr );									
OSErr EnumCache( EnumResultProcPtr resultproc, char *userDataPtr );
OSErr AddrToName( unsigned long addr, struct hostInfo *rtnStruct,
					ResultProcPtr resultproc, char *userDataPtr );									


/* OpenOurRF is called to open the MacTCP driver resources */
/* bug fix from zben@umd5.umd.edu (Ben Cranston) applied */

short OpenOurRF()
{
	SysEnvRec info;
	HParamBlockRec fi;
	Str255 filename;
	short savevol;
	OSErr status;

	SysEnvirons(1, &info);

	fi.fileParam.ioCompletion = nil;
	fi.fileParam.ioNamePtr = (StringPtr)&filename;
	fi.fileParam.ioVRefNum = info.sysVRefNum;
	fi.fileParam.ioDirID = 0;
	fi.fileParam.ioFDirIndex = 1;
	
	while (PBHGetFInfo(&fi, false) == noErr) {
		/* scan system folder for driver resource files of specific type & creator */
		if (fi.fileParam.ioFlFndrInfo.fdType == 'cdev' &&
			fi.fileParam.ioFlFndrInfo.fdCreator == 'mtcp') {
			/* found the MacTCP driver file */
				GetVol( nil, &savevol );
				SetVol( nil, info.sysVRefNum );
				status = OpenResFile( (ConstStr255Param)filename );
				SetVol( nil, savevol );
				return( status );
			}
		/* check next file in system folder */
		fi.fileParam.ioFDirIndex++;
		fi.fileParam.ioDirID = 0;
		}
	return(-1);
	}	


OSErr OpenResolver(fileName)
char *fileName;
{
	short refnum;
	OSErr rc;
	
	if (dnr != nil)
		/* resolver already loaded in */
		return(noErr);
		
	/* open the MacTCP driver to get DNR resources. Search for it based on
	   creator & type rather than simply file name */	
	refnum = OpenOurRF();
	if ( refnum != -1 ) {
		UseResFile( refnum );
	}

	/* ignore failures since the resource may have been installed in the 
	   System file if running on a Mac 512Ke */
	   
	/* load in the DNR resource package */
	codeHndl = GetIndResource('dnrp', 1);
	if (codeHndl == nil) {
		/* can't open DNR */
		if ( refnum != -1 ) {
			CloseResFile( refnum );
		}
		return(ResError());
	}
	
	DetachResource(codeHndl);
	if (refnum != -1) {
		CloseResFile(refnum);
	}
		
	/* lock the DNR resource since it cannot be reloated while opened */
	HLock(codeHndl);
	dnr = (DNRProcPtr) *codeHndl;
	
	/* call open resolver */
	rc = (*dnr)(OPENRESOLVER, fileName);
	if (rc != noErr) {
		/* problem with open resolver, flush it */
		HUnlock(codeHndl);
		DisposHandle(codeHndl);
		dnr = nil;
		}
	return(rc);
	}


OSErr CloseResolver()
{
	if (dnr == nil)
		/* resolver not loaded error */
		return(notOpenErr);
		
	/* call close resolver */
	(void) (*dnr)(CLOSERESOLVER);
	
	/* release the DNR resource package */
	HUnlock(codeHndl);
	DisposHandle(codeHndl);
	dnr = nil;
	return(noErr);
	}

OSErr StrToAddr(hostName, rtnStruct, resultproc, userDataPtr)
char *hostName;
struct hostInfo *rtnStruct;
ResultProcPtr resultproc;
char *userDataPtr;
{
	if (dnr == nil)
		/* resolver not loaded error */
		return(notOpenErr);

	return((*dnr)(STRTOADDR, hostName, rtnStruct, resultproc, userDataPtr));
}
	
OSErr AddrToStr(addr, addrStr)
unsigned long addr;
char *addrStr;									
{
	if (dnr == nil)
		/* resolver not loaded error */
		return(notOpenErr);
	
	(*dnr)(ADDRTOSTR, addr, addrStr);
	return(noErr);
	}
	
OSErr EnumCache(resultproc, userDataPtr)
EnumResultProcPtr resultproc;
char *userDataPtr;
{
	if (dnr == nil)
		/* resolver not loaded error */
		return(notOpenErr);
	return((*dnr)(ENUMCACHE, resultproc, userDataPtr));
}
	
	
OSErr AddrToName(addr, rtnStruct, resultproc, userDataPtr)
unsigned long addr;
struct hostInfo *rtnStruct;
ResultProcPtr resultproc;
char *userDataPtr;									
{
	if (dnr == nil)
		/* resolver not loaded error */
		return(notOpenErr);
	return((*dnr)(ADDRTONAME, addr, rtnStruct, resultproc, userDataPtr));
}
	