/*
 * tcpcomm.c -- TCP communication related routines
 */

#include "tcp.h"
#include "tcpinternal.h"
#include <Files.h>
#include <Devices.h>
#include <Memory.h>

static tcpstream		tcpslist[ MAXTCPS ];	/* list of open TCP streams */
static short			tcpsmax = MAXTCPS;		/* most streams we could have */
static short			drefnum = 0;			/* MacTCP driver reference number */

/*
 * local prototypes
 */
pascal void tcp_asynchnotify( StreamPtr tstream, unsigned short event, Ptr userdatap,
								unsigned short term_reason, struct ICMPReport *icmpmsg );
short		checkdriver( void );
void		bzero( char *buf, unsigned long count );

pascal void
tcp_asynchnotify( tstream, event, userdatap, term_reason, icmpmsg )
	StreamPtr			tstream;
	unsigned short		event;
	Ptr					userdatap;
	unsigned short		term_reason;	
	struct ICMPReport	*icmpmsg;
{
	tcpstream	*tsp;
	
	tsp = (tcpstream *)userdatap;
	switch( event ) {
		case TCPDataArrival:
			++(tsp->tcps_data);
			break;
		case TCPClosing:
		case TCPTerminate:
		case TCPULPTimeout:
			tsp->tcps_terminated = true;
			break;
		default:
			break;
	}
}


short
checkdriver()
{
	if ( drefnum == 0 && ( OpenDriver( "\p.IPP", &drefnum ) != noErr )) {
		return( -1 );
	}
	return( 0 );
}


/*
 * open and return an pointer to a TCP stream (return NULL if error)
 * "buf" is a buffer for receives of length "buflen."
 */
short
tcpopen( buf, buflen )
	unsigned char	*buf;
	long			buflen;
{
	TCPiopb		pb;
	OSErr		err;
	tcpstream	*tsp;
	Boolean		newtcps;
	short		s;
	
	if ( checkdriver() != 0 ) {
		return( -1 );
	}
	
	tsp = NULL;
	for ( s = 1; s < tcpsmax; ++s ) {	/* s = 0 XXX */
		if ( tcpslist[ s ].tcps_sptr == (StreamPtr)NULL ) {
			tsp = &tcpslist[ s ];
			break;
		}
	}

	if ( tsp == NULL ) {	/* out of sockets */
		return( -1 );
	}
	
	if ( buflen == 0 ) {
		buflen = TCP_BUFSIZ;
	}
	
	if ( buf == NULL && (( buf = tsp->tcps_buffer = (unsigned char *)NewPtr( buflen )) == NULL )) {
		return( -1 );
	}
	bzero( (char *)&pb, sizeof( pb ));	
	pb.csCode = TCPCreate;
	pb.ioCRefNum = drefnum;
	pb.csParam.create.rcvBuff = (Ptr) buf;
	pb.csParam.create.rcvBuffLen = buflen;
	pb.csParam.create.notifyProc = (TCPNotifyProc)tcp_asynchnotify;
	pb.csParam.create.userDataPtr = (Ptr)tsp;
		
	if (( err = PBControl( (ParmBlkPtr)&pb, 0 )) != noErr || pb.ioResult != noErr ) {
		DisposPtr( (Ptr)tsp->tcps_buffer );
		return( -1 );
	}
	
	tsp->tcps_data = 0;
	tsp->tcps_terminated = tsp->tcps_connected = false;
	tsp->tcps_sptr = pb.tcpStream;
	
	return( s );
}

/*
 * connect to remote host at IP address "addr", TCP port "port"
 * return local port assigned, 0 if error
 */
tcp_port
tcpconnect( s, addr, port )
	short			s;
	ip_addr			addr;
	tcp_port		port;
{
	tcpstream	*tsp;
	TCPiopb		pb;
	OSErr		err;

	tsp = &tcpslist[ s ];
	if ( checkdriver() != 0 || s > tcpsmax || tsp->tcps_sptr == (StreamPtr)NULL ) {
		return( 0 );
	}
		
	bzero( (char *)&pb, sizeof( pb ));	
	pb.csCode = TCPActiveOpen;
	pb.ioCRefNum = drefnum;
	pb.tcpStream = tsp->tcps_sptr;
	pb.csParam.open.remoteHost = addr;
	pb.csParam.open.remotePort = port;
	pb.csParam.open.ulpTimeoutValue = 15;
	pb.csParam.open.validityFlags = timeoutValue;
		
	if (( err = PBControl( (ParmBlkPtr)&pb, 0 )) != noErr || pb.ioResult != noErr ) {
		return( 0 );
	}
	
	tsp->tcps_connected = true;
	return( pb.csParam.open.localPort );
}


/*
 * close and release a TCP stream
 * returns 0 if no error, -1 if any error occurs
 */
short
tcpclose( s )
	short		s;
{
	TCPiopb		pb;
	tcpstream	*tsp, *tsptemp = NULL;
	short		rc;
	
	tsp = &tcpslist[ s ];
	if ( checkdriver() != 0 || s > tcpsmax || tsp->tcps_sptr == (StreamPtr)NULL ) {
		return( -1 );
	}
	
#ifdef notdef
	/*
	 * if connected execute a close
	 */
	if ( tcp->tcps_connected ) {
		bzero( (char *)&pb, sizeof( pb ));
		pb.csCode = TCPClose;
		pb.ioCRefNum = drefnum;
		pb.tcpStream = sp;
		pb.csParam.close.validityFlags = 0;
		PBControl( (ParmBlkPtr)&pb, 0 );
	}
#endif notdef
		
	bzero( (char *)&pb, sizeof( pb ));
	pb.csCode = TCPRelease;
	pb.ioCRefNum = drefnum;
	pb.tcpStream = tsp->tcps_sptr;
	pb.csParam.close.validityFlags = 0;
	rc = 0;
	
	if ( PBControl( (ParmBlkPtr)&pb, 0 ) != noErr || pb.ioResult != noErr ) {
		rc = -1;
	}
	
	DisposPtr( (Ptr)tsp->tcps_buffer );
	tsp->tcps_sptr = (StreamPtr)NULL;		/* mark as no longer in use */
		
	return( rc );	
}


/*
 * wait for new data to arrive on socket "s"
 *
 * if "timeout" is NULL, wait until data arrives or connection goes away
 * if "timeout" is a pointer to a zero'ed struct, poll
 * else wait for "timeout->tv_sec + timeout->tv_usec" seconds
 */
short
tcpselect( s, timeout )
	short			s;
	struct timeval	*timeout;
{
	tcpstream		*tsp;
	long			ticks, endticks;

	tsp = &tcpslist[ s ];
	if ( checkdriver() != 0 || s > tcpsmax || tsp->tcps_sptr == (StreamPtr)NULL ) {
		return( -1 );
	}
	
	if ( timeout != NULL ) {
		endticks = 60 * timeout->tv_sec + ( 60 * timeout->tv_usec ) / 1000000 + TickCount();
	}

	ticks = 0;	
	while (( timeout == NULL || ticks < endticks ) && tsp->tcps_data < 1 && !tsp->tcps_terminated ) {
		Delay( 1L, &ticks );
	} 
	
	if ( tsp->tcps_terminated ) {
		return( -1 );
	}
	
	return ( tsp->tcps_data < 1 ? 0 : 1 );
}


/*
 * read up to "rbuflen" bytes into "rbuf" from a connected TCP stream
 * wait up to "timeout" seconds for data (if timeout == 0, wait "forever")
 */
long
tcpread( s, timeout, rbuf, rbuflen, dlp )
	short			s;
	byte			timeout;
	unsigned char	*rbuf;
	unsigned short	rbuflen;
	DialogPtr		dlp;
{
	TCPiopb			pb;
	tcpstream		*tsp;
	unsigned long	time, end_time;
	
	tsp = &tcpslist[ s ];
	if ( checkdriver() != 0 || s > tcpsmax || tsp->tcps_sptr == (StreamPtr)NULL ) {
		return( -1 );
	}
	
	GetDateTime( &time );
	end_time = time + timeout;
/*	ShowCursor();
	Rotate_Cursor( 0 );
*/

	while(( timeout <= 0 || end_time > time ) && tsp->tcps_data < 1 &&
			!tsp->tcps_terminated ) {
/*		if ( item_one_hit( dlp )) {
			return( -2 );
		}
		Spin_Cursor( +SPIN_RATE );
*/		Delay( 1L, (long *)&time );
		GetDateTime( &time );
	}
	
	if ( tsp->tcps_data < 1 ) {
		return( tsp->tcps_terminated ? -3 : -1 );
	}
		
	bzero( (char *)&pb, sizeof( pb ));	
	pb.csCode = TCPRcv;
	pb.ioCRefNum = drefnum;
	pb.tcpStream = tsp->tcps_sptr;
	pb.csParam.receive.commandTimeoutValue = timeout;
	pb.csParam.receive.rcvBuff = (char *)rbuf;	
	pb.csParam.receive.rcvBuffLen = rbuflen;
	
	if ( PBControl( (ParmBlkPtr)&pb, 0 ) != noErr || pb.ioResult != noErr ) {
		return( -1 );
	}
	
	if ( --(tsp->tcps_data) < 0 ) {
		tsp->tcps_data = 0;
	}
	
	return( pb.csParam.receive.rcvBuffLen );
}
	

/*
 * send TCP data
 * "sp" is the stream to write to
 * "wbuf" is "wbuflen" bytes of data to send
 * returns < 0 if error, number of bytes written if no error
 */
short
tcpwrite( s, wbuf, wbuflen )
	short			s;
	unsigned char	*wbuf;
	unsigned short	wbuflen;
{
	tcpstream	*tsp;
	TCPiopb		pb;
	wdsEntry	wds[ 2 ];
	OSErr 		err;
	
	tsp = &tcpslist[ s ];
	if ( checkdriver() != 0 || s > tcpsmax || tsp->tcps_sptr == (StreamPtr)NULL ) {
		return( -1 );
	}
	
	wds[ 0 ].length = wbuflen;
	wds[ 0 ].ptr = (char *)wbuf;
	wds[ 1 ].length = 0;
	
	bzero( (char *)&pb, sizeof( pb ));	
	pb.csCode = TCPSend;
	pb.ioCRefNum = drefnum;
	pb.tcpStream = tsp->tcps_sptr;
	pb.csParam.send.wdsPtr = (Ptr)wds;
	pb.csParam.send.validityFlags = 0;
	
	if (( err = PBControl( (ParmBlkPtr)&pb, 0 )) != noErr || pb.ioResult != noErr ) {
		return( -1 );
	}
	
	return( wbuflen );
}


/*
 * bzero -- set "len" bytes starting at "p" to zero
 */
static void
bzero( p, len )
 	char			*p;
	unsigned long	len;
{
	unsigned long	i;
	
	for ( i = 0; i < len; ++i ) {
		*p++ = '\0';
	}
}

pascal void setdoneflag( struct hostInfo *hip, char *donep );


pascal void
setdoneflag( hip, donep )
 	struct hostInfo *hip;
	char			*donep;
{
	++(*donep);
}


/*
 * return a hostInfo structure for "host" (return NULL if error)
 */
struct hostInfo *
gethostinfobyname( host )
 	char *host;
{
 	static char 			open = 0;
	static struct hostInfo	hi;
	char					done = 0;
	OSErr					err;
	long					time;
	
	/*
	 * if this is the first query, initialize the resolver routines
	 */
	if ( !open ) {
		if (( err = OpenResolver( NULL )) != noErr ) {
			return( NULL );
		}
		++open;
	}
/*	ShowCursor();
*/
	
	if (( err = StrToAddr( host, &hi, (ResultProcPtr)setdoneflag, &done ))  == cacheFault ) {
		/*
		 * 	querying DN servers; wait for reply
		 */
		while( !done ) {
			Delay( 1, &time );
/*			Spin_Cursor( +SPIN_RATE);
*/
		}
		err = hi.rtnCode;
	}
	
/*	SetCursor( &arrow );
*/
	/*
	 * check for error not caught error in StrToAddr()
	 */
	if ( err != noErr || ( ( err == noErr ) && ( hi.addr[ 0 ] == 0 ))) {
		return( NULL );
	}
	
	return( &hi );
}
 