/*
 * Copyright (c) 1991, 1992 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <lber.h>
#include <ldap.h>
#include "ud.h"

char me[BUFSIZ];		/* the user's set name */
char password[512];		/* password entered my user */
char bound_as[BUFSIZ];		/* the user's bound name */
int verbose;			/* verbosity indicator */
extern LDAP *ld;		/* our LDAP descriptor */
extern int debug;		/* debug flag */


bind_info()
{
	printf("  Bound as \"%s\"\n", bound_as);
}

auth(who, best_guess)
char *who, *best_guess;
{
	int rc;			/* return code from ldap_bind() */
	char *passwd;		/* returned by mygetpass() */
	char *dn;		/* distinghished name from ldap_get_dn() */
	char **rdns;		/* for fiddling with the DN */
	LDAPMessage *mp;	/* returned from find() */
	static char prompt[BUFSIZ];	/* place for us to sprintf the prompt */
	static char name[BUFSIZ];	/* place to store the user's name */
	extern struct entry Person;	/* look here for a name if needed */
	extern char *mygetpass();	/* getpass() passwds are too short */
	extern LDAPMessage *find();	/* for looking up 'name' */

	if (debug)
		fprintf(stderr, "auth(%s,%s)\n", who, best_guess);

	if (who == NULL) {
		/*
		 *  The user needs to bind.  If <who> is not specified, we
		 *  can make some guesses.  If the user has used the 'iam'
		 *  command, then try the name s/he set.  If not, try the
		 *  name retrieved with the last 'find' operation.  If no
		 *  such command has been issued, we don't have any guess,
		 *  and the user will have to supply a name.
		 */
		if (me[0] != '\0')
			who = me;
		else if (Person.name[0] != '\0')
			who = Person.name;
		else if (best_guess != NULL)
			who = best_guess;
		do {
			if (who == NULL) {
				printf("  What is your name? ");
				fflush(stdout);
				fetch_buffer(name, sizeof(name), stdin);
				if (name[0] != '\0')
					who = name;
			}
			else {
				printf("  Is your name \"%s\"? ", who);
				fflush(stdout);
				fetch_buffer(name, sizeof(name), stdin);
				if ((name[0] != '\0') && strncasecmp(name, "yes", strlen(name)))
					who = NULL;
					/* and loop again */
			}
		} while (who == NULL);
	}

	/*
	 *  Bail out if the name is bogus.  If not, strip off the junk
	 *  at the start of the DN, build a prompt, and get a password 
	 *  from the user.  Then perform the ldap_bind().
	 */
	if ((mp = find(who)) == NULL) {
		(void) ldap_msgfree(mp);
		return(0);
	}
	dn = ldap_get_dn(ld, ldap_first_entry(ld, mp));
	(void) ldap_msgfree(mp);
	rdns = ldap_explode_dn(dn, TRUE);

	sprintf(prompt, "  Enter your password: ");
	do {
		passwd = mygetpass(prompt);
	} while (passwd != NULL && *passwd == '\0');
	if (passwd == NULL) {
		(void) ldap_value_free(rdns);
		return(0);
	}
	rc = ldap_bind(ld, dn, passwd, UD_AUTH_METHOD);
	if (rc != LDAP_SUCCESS) {
		if (ld->ld_errno == LDAP_NO_SUCH_ATTRIBUTE)
			fprintf(stderr, "  Entry has no password\n");
		else if (ld->ld_errno == LDAP_INVALID_CREDENTIALS)
			fprintf(stderr, "  Password does not match\n");
		else
			ldap_perror(ld, "ldap_bind" );
		(void) ldap_value_free(rdns);
		return(-1);
	}
	else if (verbose)
		printf("  Authentication successful.\n\n");
	else
		printf("\n");
	(void) strcpy(bound_as, *rdns);
	(void) strcpy(password, passwd);
	(void) ldap_value_free(rdns);
	return(0);
}

iam(who)
char *who;
{
	if (!strcmp(who, "me"))
		printf("  Of course you are.\n");
	else {
		(void) strcpy(me, who);
		print_my_name();
	}
}

print_my_name()
{
	if (me[0] == '\0')
		printf("  I do not know who you are.  Use IAM to tell me.\n");
	else
		printf("  You are \"%s\".\n", me);
}
