/*
 * Copyright (c) 1992 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <lber.h>
#include <ldap.h>
#include "ud.h"

print_help(s)
char *s;
{
	char *cp;		/* misc char ptr */
	char cmd[BUFSIZ];	/* buffer to hold command */
	int len;			/* command length */

	cp = cmd;
	while (!isspace(*s) && (*s != '\0')) {
		*cp++ = *s++;
	}
	*cp = '\0';
	len = strlen(cmd);

	/* print general help, or just on topic 's' if provided */
	if ((cmd[0] == '\0') || !strncasecmp("commands", cmd, (len > 2) ? len : 2)) {
		printf("\n  Here are brief descriptions of available commands:\n\n");
		printf("  ?             To print this list.\n");
		printf("  bind          To bind (authenticate) to the Directory.\n");
		printf("  cb            To change the search base.\n");
		printf("  change        To change information associated with an entry.\n");
		printf("  find          To find an entry in the Directory\n");
		printf("  help          To print this list.\n");
		printf("  iam           To set your name while running the program.\n");
		printf("  quit          To terminate the program.\n");
		printf("  verbose       To toggle the verbose switch.\n");
		printf("  whoami        To print who you are (used with 'iam').\n");

		printf("\n  Type \"help <command-name>\" to get help about a particular command.\n\n");
	}
	else if (!strncasecmp("status", cmd, len)) {
		printf("  status\n\n");
		format("Prints out the current connection status.  Lists the name of the current DIXIE server, the current search base, and the identity to which you are bound.  If you have not bound as anyone then ud considers you bound as Nobody.  cd is an alias for cb.", 75);
	}
	else if (!strncasecmp("cd", cmd, len) || !strncasecmp("cb", cmd,len)) {
		printf("  cb <new-base>\n");
		printf("  cd <new-base>\n\n");
		format("The cb command changes the search base.  By default, this program looks only in the local part of the Directory.  By using the cb command, you can search other parts of the Directory.", 75);
	printf("\n  Examples:\n");
	printf("\n            * cb ..\n\n");
	format("changes the search base so that it is one level higher in the Directory.  Note that if you perform several of these in a row you will move to the root of the Directory tree.", 75);
	printf("\n            * cb ?\n\n");
	format("prints out a list of the possible areas to search below the current search base.  This is useful once you have moved high in the tree and wish to snoop about.", 75);
	printf("\n            * cb default\n\n");
	format("sets the search base to its original default value.", 75);
	printf("\n            * cb o=Merit Computer Network, c=US\n\n");
	format("sets the search base to organization given, the Merit Computer Network in this case.  This comamnd checks the validity of the specified search base, and rejects it if it is not a valid Distinguished Name (DN).  A DN uniquely identifies a portion of the global X.500 namespace.", 75);
	}
	else if (!strncasecmp("quit", cmd, len) || !strncasecmp("stop",cmd, len)) {
		printf("  quit\n");
		printf("  stop\n\n");
		printf("  Quits the program.  'stop' is an alias for 'quit'.\n");
	}
	else if (!strncasecmp("find", cmd, len) || !strncasecmp("display", cmd, len) || !strncasecmp("show", cmd, len)) {
		printf("  find <who>\n");
		printf("  show <who>\n");
		printf("  display <who>\n\n");
		format("Displays information about the person specified.  If the name specified matches more than one person, one will be presented a list from which to make a choice.  'show' and 'display' are aliases for 'find.'", 75);
	}
	else if (!strncasecmp("bind", cmd, len)) {
		printf("  bind [who]\n\n");
		format("Binds (authenticates) to the Directory.  It is generally necessary to bind to the Directory in order to look at privileged entries or to modify an entry.   Allows one to authenticate prior to issuing a 'change' or 'modify' command.  Most often used by administrators to bind to an identity.", 75);
	}
	else if (!strncasecmp("modify", cmd, len) || !strncasecmp("change", cmd, len)) {
		printf("  modify [who]\n");
		printf("  change [who]\n\n");
		format("Changes information associated with an entry in the X.500 Directory.  'change' is an alias for 'modify'.", 75);
	}
	else if (!strncasecmp("iam", cmd, len)) {
		printf("  iam <name>\n\n");
		format("Sets your name so that the word 'me' may be used as a legal identifier when issuing other commands.  For instance, after issuing a 'iam John Doe' command, one could then issue a 'display me' command instead of having to type 'display John Doe.'", 75);
	}
	else if (!strncasecmp("whoami", cmd, len)) {
		printf("  whoami\n\n");
		format("Prints the name set with the 'iam' command.", 75);
	}
	else if (!strncasecmp("verbose", cmd, len)) {
		printf("  verbose\n\n");
		format("Turns on long and windy messages which might be useful to new users of this program.  If verbose mode is already on, this turns it off.", 75);
	}
	else if (!strncasecmp("help", cmd, len)) {
		format("Prints out a brief description of each command.", 75);
	}
	else if (cmd[0] == '?') {
		format("Prints out a brief description of each command.  Same as typing 'help help'.", 75);
	}
	else {
		printf("  Don't recognize <%s>\n", cmd);
	}
}

format(s, width)
char *s;
int width;
{
	char c;
	register char *cp;

	for (;;) {
		if (strlen(s) + 2 < width) {
			printf("  %s\n", s);
			return; 
			/*NOTREACHED*/
		}
		cp = s + width - 2;
		while (!isspace(*cp) && (cp != s))
			cp--;
		c = *cp;
		*cp = '\0';
		printf("  %s\n", s);
		s = cp + 1;
		*cp = c;			/* don't mess up 's' */
	}
}
