/*
 * Copyright (c) 1991 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <lber.h>
#include <ldap.h>
#ifndef __STDC__
#include <memory.h>
#endif
#include <sys/types.h>
#include <malloc.h>
#include "ud.h"

extern struct entry Person; 
extern int verbose;
extern int debug;
extern LDAP *ld;

modify(who)
char *who;
{
#ifdef UOFM
	void set_updates();	/* routine to modify noBatchUpdates */
#endif
	void set_pw();		/* routine to modify a password */
	extern char bound_as[]; /* who this user is bound as */
	extern char me[];	/* set if user has used I AM */
	LDAPMessage *mp;	/* returned from find() */
	char *dn;		/* distinguished name */
	char **rdns;		/* for fiddling with the DN */
	char name[BUFSIZ];	/* entry to modify */
	static char ans[RESP_SIZE];	/* for holding user input */
	extern LDAPMessage *find();	/* for finding the person to modify */
#ifdef UOFM
	static char printed_warning = 0;	/* for use with the */
	extern char * fetch_update_setting();	/* noBatchUpdates attr */
#endif

	/*
	 *  First, decide what entry we are going to modify.  If the
	 *  user has not included a name on the modify command line,
	 *  we will use the person who was last looked up with a find
	 *  command.  If there is no value there either, we don't know
	 *  who to modify.
	 *
	 *  Once we know who to modify, be sure that they exist, and
	 *  parse out their DN.
	 */
	if ((who == NULL) && (Person.name[0] == '\0') && !strcmp(bound_as, "Nobody") && (me[0] == '\0')) {
		printf("  Modify who? ");
		fflush(stdout);
		fetch_buffer(name, sizeof(name), stdin);
		if (name[0] != '\0')
			who = name;
		else
			return;
	}
	else if ((who == NULL) && (Person.name[0] != '\0'))
		who = Person.name;
	else if ((who == NULL) && (me[0] != '\0'))
		who = me;
	else if ((who == NULL) && strcmp(bound_as, "Nobody"))
		who = bound_as;
	if ((mp = find(who)) == NULL) {
		(void) ldap_msgfree(mp);
		return;
	}
	dn = ldap_get_dn(ld, ldap_first_entry(ld, mp));
	rdns = ldap_explode_dn(dn, TRUE);
	if (debug)
		printf("  Modifying entry for \"%s\"\n", *rdns);

	/* bind if necessary */
	if (!strcmp(bound_as, "Nobody")) {
		if (auth(who, who) < 0) {
			(void) ldap_msgfree(mp);
			(void) free(dn);
			return;
		}
	}

	if (debug)
		fprintf(stderr, "Done binding.\n");

#ifdef UOFM
	/*
	 *  If verbose mode is turned on and the user has not set a value
	 *  for noBatchUpdates, warn them that what they are about to do
	 *  may be overwritten automatically by that Stinkbug.
	 */
	(void) fetch_update_setting(dn);
	if (verbose && !printed_warning && (ld->ld_errno == LDAP_NO_SUCH_ATTRIBUTE)) {
		printed_warning = 1;
		printf("\n  WARNING!\n");
		printf("  You are about to make a modification to an X.500 entry\n");
		printf("  that has its \"automatic updates\" field set to ON.\n");
		printf("  This means that the entry will be automatically updated\n");
		printf("  each month from official University sources like the\n");
		printf("  Personnel Office.  With \"automatic updates\" set to ON,\n");
		printf("  the following fields will be overwritten each month:\n");
		printf("         Title, home address and phone,\n");
		printf("         business address and phone\n");
		printf("  If you modify any of these fields, you may want to change\n");
		printf("  the \"automatic updates\" field to OFF so that your\n");
		printf("  changes will not be overwritten.  You may change this\n");
		printf("  setting by choosing \"u\" at the \"Modify what?\" prompt\n");
	}
#endif

	/*
	 *  Current values for user 'who' are being held in 'mp'. We
	 *  should parse up that buffer and fill in the Person structure.
	 *  Once we're done with that, we can find out which fields the
	 *  user would like to modify.
	 */
	parse_answer(mp);
	(void) ldap_msgfree(mp);
	if (verbose) {
		printf("  You now need to specify what field you'd like to modify.\n\n");
			printf("  Choices are:\n");
			printf("  -----------------------\n");
			printf("  e  E-mail address\n");
			printf("  t  Title\n");
			printf("  p  Home phone\n");
			printf("  P  Business phone\n");
			printf("  a  Home address\n");
			printf("  A  Business address\n");
			printf("  f  Fax number\n");
			printf("  d  Description\n");
			printf("  x  Password\n");
#ifdef UOFM
			printf("  u  Automatic updates\n");
#endif
			printf("  m  Member list (for groups)\n");
			printf("\n  Hitting RETURN will abort the process.\n\n");
	}
again:
	printf("  Modify what? ");
	fflush(stdout);
	fetch_buffer(ans, sizeof(ans), stdin);
	switch (ans[0]) {
	case 'h' :
	case 'H' :
	case '?' :
		printf("\n  Choices are:\n");
		printf("  -----------------------\n");
		printf("  e  E-mail address\n");
		printf("  t  Title\n");
		printf("  p  Home phone\n");
		printf("  P  Business phone\n");
		printf("  a  Home address\n");
		printf("  A  Business address\n");
		printf("  f  Fax number\n");
		printf("  d  Description\n");
		printf("  x  Password\n");
#ifdef UOFM
		printf("  u  Automatic updates\n");
#endif
		printf("  m  Member list (for groups)\n");
		printf("  ?  Print this list\n\n");
		goto again;
		/* NOTREACHED */
	case 't' :
		change_field("Title", TITLE, Person.title, dn);
		break;
	case 'e' :
		change_field("E-mail address", E_MAIL_ADDRESS, Person.email, dn);
		break;
	case 'p' :
		change_field("Home phone", HOME_PHONE, Person.hphon, dn);
		break;
	case 'P' :
		change_field("Business phone", PHONE_NUMBER, Person.bphon, dn);
		break;
	case 'a' :
		change_field("Home address", HOME_ADDRESS, Person.haddr, dn);
		break;
	case 'A' :
		change_field("Business address", POSTAL_ADDRESS, Person.baddr, dn);
		break;
	case 'f' :
		change_field("Fax number", FAX, Person.faxxx, dn);
		break;
	case 'm' :
		change_field("Member list", MEMBERS, Person.mmbrs, dn);
		break;
	case 'd' :
		change_field("Description", MLDESCRIPTION, Person.mldsc, dn);
		break;
	case 'x' :
		set_pw(dn);
		break;
#ifdef UOFM
	case 'u' :
	case 'U' :
		set_updates(dn);
		break;
#endif
	case '\0' :
		putchar('\n');
		break;
	default :
		printf("  Unrecognizable option\n\n");
		goto again;
		/* NOTREACHED */
	}
	(void) free(dn);
	return;
}

void set_pw(who)
char *who;
{
	char *cp;
	static char new[513];
	extern char * mygetpass();
	extern char password[];
	static char *newpasswd[2];
	static LDAPMod mod = { LDAP_MOD_REPLACE, PASSWORD, newpasswd };
	static LDAPMod *mods[2] = { &mod };

	if (verbose) {
		printf("  You indicated that you wanted to change your password.\n");
		printf("  You will now be prompted for your old, existing password.\n");
		printf("  Then you will be prompted twice for your new password.\n");
		printf("  If they match, and your old password is correct, your\n");
		printf("  password will be changed.\n\n");
	}
	cp = mygetpass("  Old password? ");
	if (strcmp(cp, password)) {
		printf("  Incorrect password -- not changed.\n");
		return;
	}
	cp = mygetpass("  New password? ");
	(void) strcpy(new, cp);
	cp = mygetpass("  Enter new password again to confirm: ");
	if (strcmp(cp, new)) {
		printf("  Password mismatch -- not changed.\n");
		return;
	}
	newpasswd[0] = cp;
	if (ldap_modify_s(ld, who, mods)) {
		ldap_perror(ld, "ldap_modify_s");
		return;
	}
	else if (verbose)
		printf("  Password changed.\n");
	(void) strcpy(password, cp);
	return;
}


/* generic routine for changing any field */
change_field(tag, id, val, who)
char *tag;			/* user-friendly tag identifying the field */
char *id;			/* X.500 internal name for the field */
char val[][MAX_VAL_SIZE];	/* current set of values for this field */
char *who;			/* DN of user whose entry we are changing */
{
				
	char *get_value();		/* routine to extract values */
	static char buf[4 * RESP_SIZE];	/* for printing things */
	static char resp[RESP_SIZE];	/* for user input */
	char *prompt, *prompt2, *more;
	register int i;				/* for looping thru values */
	static LDAPMod mod;
	static LDAPMod *mods[2] = { &mod };	/* passed to ldap_modify */
	static char *values[MAX_VALUES];	/* passed to ldap_modify */

	/*
	 *  If there is no current value associated with the attribute,
	 *  then this is the easy case.  Collect one (or more) attributes
	 *  from the user, and then call ldap_modify_s() to write the changes
	 *  to the DIXIE server.
	 */
	for (i = 0; i < MAX_VALUES; i++)
		values[i] = NULL;
	if (val[0][0] == '\0') {
		printf("\n  No current \"%s\"\n", tag);
		values[0] = get_value(id, "Enter a value");
		mod.mod_op = LDAP_MOD_REPLACE;
		mod.mod_type = id;
		mod.mod_values = values;
		for (i = 1; i < MAX_VALUES; i++) {
			printf("  Do you wish to add an additional value? ");
			fflush(stdout);
			fetch_buffer(resp, sizeof(resp), stdin);
			if ((resp[0] == 'y') || (resp[0] == 'Y'))
				values[i] = get_value(id, "Enter an additional value");
			else
				break;
		}
		if (ldap_modify_s(ld, who, mods) < 0) {
			ldap_perror(ld, "ldap_modify_s");
			return;
		}
		else if (verbose)
			printf("  Modification of '%s' complete.\n", tag);
		for (i--; i > 0; i--)
			(void) free(values[i]);
	}
	/*
	 *  There are values for this attribute already.  In this case,
	 *  we want to allow the user to delete all existing values,
	 *  add additional values to the ones there already, or just
	 *  delete some of the values already present.  DIXIE does not
	 *  handle modifications where the attribute occurs on the LHS
	 *  more than once.  So to delete entries and add entries, we
	 *  need to call ldap_modify() twice.
	 */
	else {
		sprintf(buf, "%s:  ", tag);
		print_entry(val, (char *) buf);
		if (verbose) {
			printf("  You may now:\n");
			printf("    Add additional values to the existing ones, OR\n");
			printf("    Clear all values listed above, OR\n");
			printf("    Delete one of the values listed above, OR\n");
			printf("    Replace all of the values above with new ones.\n");
		}
		printf("\n  Add, Clear, Delete, or Replace? ");
		fflush(stdout);
		fetch_buffer(resp, sizeof(resp), stdin);

		/*
		 *  If the want to clear the values, just do it.
		 */
		mod.mod_type = id;
		mod.mod_values = values;
		if ((resp[0] == 'c') || (resp[0] == 'C')) {
			mod.mod_op = LDAP_MOD_DELETE;
			mod.mod_values = NULL;
			if (debug) {
				printf("Clearing attribute '%s'\n", id);
				printf("who = [%s]\n", who);
				printf("mod = [%d] [%s] [%x]\n", mod.mod_op,
					mod.mod_type, mod.mod_values);
			}
			if (ldap_modify_s(ld, who, mods) < 0) {
				ldap_perror(ld, "ldap_modify_s");
				return;
			}
			else if (verbose)
				printf("  '%s' has been cleared.\n", tag);
			return;
		}

		if ((resp[0] == 'a') || (resp[0] == 'A')) {
			prompt = "Enter the value you wish to add";
			more = "  Add an additional value? ";
			prompt2 = "Enter another value you wish to add";
			mod.mod_op = LDAP_MOD_ADD;
		}
		else if ((resp[0] == 'd') || (resp[0] == 'D')) {
			prompt = "Enter the value you wish to delete";
			more = "  Delete an additional value? ";
			prompt2 = "Enter another value you wish to delete";
			mod.mod_op = LDAP_MOD_DELETE;
		}
		else if ((resp[0] == 'r') || (resp[0] == 'R')) {
			prompt = "Enter the new value";
			more = "  Add an additional value? ";
			prompt2 = "Enter another value you wish to add";
			mod.mod_op = LDAP_MOD_REPLACE;
		}
		else
			printf("  No changes made.\n");

		values[0] = get_value(id, prompt);
		for (i = 1; i < MAX_VALUES; i++) {
			printf(more);
			fflush(stdout);
			fetch_buffer(resp, sizeof(resp), stdin);
			if ((resp[0] == 'y') || (resp[0] == 'Y'))
				values[i] = get_value(id, prompt2);
			else
				break;
		}
		if (ldap_modify_s(ld, who, mods) < 0) {
			ldap_perror(ld, "ldap_modify_s");
			return;
		}
		else if (verbose)
			printf("  Modifications to '%s' complete.\n", tag);
		for (i--; i > 0; i--)
			(void) free(values[i]);
	}
}

char *get_value(id, prompt)
char *id, *prompt;
{
	char *cp;		/* for the malloc() */
	int count;		/* line # of new value -- if multiline */
	int multiline = 0;	/* 1 if this value is multiline */
	static char line[LINE_SIZE];	/* raw line from user */
	static char buffer[MAX_LINES * LINE_SIZE];	/* holds ALL of the 
							   lines we get */

	/*
	 *  To start with, we have one line of input from the user.
	 *
	 *  Addresses and multiline description can span multiple lines.
	 *  Other attributes may not.
	 */
	count = 1;
	(void) memset(buffer, 0, sizeof(buffer));
	if (!strcmp(id, POSTAL_ADDRESS) || !strcmp(id, HOME_ADDRESS) || !strcmp(id, MLDESCRIPTION))
		multiline = 1;
	printf("\n  %s:\n", prompt);

	/* fetch lines */
	for (;;) {
		if (multiline)
			printf(" %1d: ", count);
		else
			printf("  > ");
		fflush(stdout);
		fetch_buffer(line, sizeof(line), stdin);
		if (line[0] == '\0')
			break;

		/*
		 *  Separate lines of multiline attribute values with
		 *  dollar signs.  Copy this line into the buffer we
		 *  use to collect up all of the user-supplied input
		 *  lines.  If this is not a multiline attribute, we
		 *  are done.
		 */
		if (count++ > 1)
			(void) strcat(buffer, "$");
		(void) strcat(buffer, line);
		if (!multiline)
			break;
	}
	if (debug)
		printf("  Value is [%s]\n", buffer);
	if ((cp = (char *) malloc((unsigned) strlen(buffer))) == NULL)
		fatal("malloc");
	strcpy(cp, buffer);
	return(cp);
}

#ifdef UOFM

void set_updates(who)
char *who;
{
	char *cp, *s;
	extern char * fetch_update_setting();
	static char response[16];

	static char *newsetting[2];
	static LDAPMod mod = { LDAP_MOD_REPLACE, UPDATES, newsetting };
	static LDAPMod *mods[2] = { &mod, NULL };
	static char value[MAX_VAL_SIZE];

	/* explain what the implications are */
	if (verbose) {
		printf("\n  By default, updates that are received from the Personnel\n");
		printf("  Office and the Office of the Registrar are applied to all\n");
		printf("  entries in the X.500 database each month.  Sometimes this\n");
		printf("  feature is undesirable.  For example, if you maintain your\n");
		printf("  entry in the X.500 database manually, you may not want to\n");
		printf("  have these updates applied to your entry, possibly overwriting\n");
		printf("  correct information with out-dated information.\n\n");
	}

	/* fetch the current setting */
	if ((cp = fetch_update_setting(who)) == NULL)
		return;
	if (!strcmp(cp, "TRUE"))
		printf("  Automatic updates are currently turned OFF\n");
	else if (!strcmp(cp, "FALSE"))
		printf("  Automatic updates are currently turned ON\n");
	else {
		fprintf(stderr, "  Unknown update flag -> [%s]\n", cp);
		return;
	}

	/* see if they want to change it */
	printf("\n  Change this setting [no]? ");
	fflush(stdout);
	(void) fgets(response, sizeof(response), stdin);
	for (s = response; isspace(*s); s++)
			;
	if ((*s == 'y') || (*s == 'Y')) {
		if (!strcmp(cp, "TRUE"))
			strcpy(value, "FALSE");
		else
			strcpy(value, "TRUE");
		if (ldap_modify_s(ld, who, mods) < 0) {
			ldap_perror(ld, "ldap_modify_s");
			return;
		}
		else if (verbose)
			printf("  Setting has been changed\n");
		return;
	}
	if (verbose)
			printf("  Setting has not been changed\n");
}

#endif
